Attribute VB_Name = "ReadWrite"
' FUNCTIONS:
' InitialiseADCInstructionStream()
' InitialiseDACInstructionStream()
' ReadVoltage(Pin As Integer, DataLength As Byte, LSBF As Byte, BIP As Byte) As Double
' WriteVoltage(Pin As Byte, Range As Byte, Voltage As Double)
' PrintADC
' PrintADCResult
' PrintDAC


Option Explicit

Public Const LO = 0
Public Const HI = 1

'Port adresses
Public Const DATA = &H378
Public Const STATUS = DATA + 1


'Assign pin numbers

' DATA port
Public Const CS = 0         ' ADC Chip Select.
Public Const LOAD = 0       ' DAC Chip select. ADC CS and DAC LOAD share same pin.
Public Const ADCCLK = 1     ' ADC clock.
Public Const DACCLK = 2     ' DAC clock.
Public Const DATAIN = 3     ' ADC and DAC instruction data share same pin.

' STATUS port
Public Const DATAOUT = 6    ' ADC result returned on this STATUS port pin ( nACK ).

' Reference voltages.
Public Const ADCREF = 5  ' ADC Reference voltage.
Public Const DACREF = 2.5 ' DAC Reference voltage.

' Instruction streams.
Private ADCInstructionStream(36) As Byte ' ADC output data stream.
Private DACInstructionStream(37) As Byte ' DAC output data stream.

Public ADCResult(12) As Byte ' 12 byte ADC input result stream.


' InitialiseADCInstructionStream : Initialise the ADC instruction byte array with clock pulses and setting CS=LO.

Sub InitialiseADCInstructionStream()

Dim n As Integer


    For n = 0 To 35 Step 3
    
        ADCInstructionStream(n) = 0
                
        ADCInstructionStream(n + 1) = 0
        SetBit ADCInstructionStream(n + 1), ADCCLK, HI
        
        ADCInstructionStream(n + 2) = 0
    
    Next
    
End Sub


' InitialiseDACInstructionStream : Initialise the DAC byte array with clock pulses and setting LOAD(CS)=HI.

Sub InitialiseDACInstructionStream()

Dim n As Integer
    
    
    For n = 0 To 35 Step 3

        DACInstructionStream(n) = 0
        SetBit DACInstructionStream(n), LOAD, HI 'Select DAC

        DACInstructionStream(n + 1) = 0
        SetBit DACInstructionStream(n + 1), LOAD, HI 'Select DAC
        SetBit DACInstructionStream(n + 1), DACCLK, HI

        DACInstructionStream(n + 2) = 0
        SetBit DACInstructionStream(n + 2), LOAD, HI 'Select DAC

    Next
    
    ' Set LOAD to 'LO' to tell the DAC to output the voltage.
    DACInstructionStream(36) = 0
    
End Sub


' ReadVoltage : Read a voltage from one of 11 ADC input pins.

'Params :-
'Pin        : Specify the input pin from which to read a voltage ( 0-10 ).
'DataLength : Specify the length of the instruction/conversion result.
'LSBF       : Specify whether the least significant bit (LSB) or most significant bit (MSB)
'             is shifted out first, i.e. the direction that the result is to be shifted out.
'BIP        : Specifiy whether the result is to be a uni-polar or bi-polar convesrion.


'********************************************************************************************************
'
' ADC control lines: CS( Chip Select=LO ),CLK,DATAIN,DATAOUT
'
' DATA = 888    LPT1 port address ( OUT: Control info to ADC i.e, CS,ADCCLK,DATAIN )
' STATUS = 889  STATUS port address ( IN: Retrieve result from ADC using ACK pin 6 i.e, DATAOUT )
'
' ADC 12-bit instruction format: A3,A2,A1,A0,L1,L0,LSFB,BIP,0,0,0,0 fed to ADC using DATAIN line.
' A=Address, L=Instruction length,LSBF=Least Significant Bit First,BIP=Bi-Polar result
'
' For example, an instruction sent to the ADC is: 0,0,0,0,0,0,0,0,0,0,0,0
'
'               Address(4 bits)= 0,0,0,0 ( equates to ADC input pin 0. This pin has 2.5V presented to it)
'               Instruction Length(2 bits)= 0,0 ( equates to an instruction length of 12 bits )
'               LSBF(1 bit)= 0 ( equates to outputting most significant bit first )
'               BIP(1 bit)= 0 ( equates to no bi-bipolar result )
'               Pad bits(4 bits)= 0,0,0,0 ( pads out the instruction length to the required 12 bits
'                                         so that the input and the simultaneous output result of
'                                         12-bits resolution can be synchronised )
'
'********************************************************************************************************
'
' DATA port output byte format: 0,0,0,0,DATAIN,0,ADCCLK,CS
'
' Each instruction bit sent to the ADC on DATAIN consists of a 3 byte write sequence
' that encapsulates a full CLK pulse. This is to ensure that each instruction bit
' is present on the DATAIN line BEFORE and AFTER a CLK pulse is generated.
' This is to eliminate any timing issues so that an instruction bit is properly
' captured by the ADC.
'
'**********************************************************************************************
'
' STATUS port input byte format: 0,DATAOUT,0,0,0,0,0,0
'
' The digitally converted 12-bit result from the previous instruction is shifted out
' as the current instruction is clocked in. The MSB of the result of the previous
' conversion should already be present on the DATAOUT line ( bit 6 of the STATUS port).
'
'**********************************************************************************************


Function ReadVoltage(Pin As Byte, DataLength As Byte, LSBF As Byte, BIP As Byte) As Double

Dim BitValue As Byte
Dim iADCResult As Integer
Dim dVoltage As Double
Dim n As Integer
Dim iIndex As Integer


    'Convert the input parameters to an ADC instruction sequence by amending the
    'previously initialised ADC data stream.
    
    'Set address ( 4 bits ).
    For n = 0 To 11 Step 3
        BitValue = ChkBit(Pin, 3 - n / 3)
        SetBit ADCInstructionStream(n), DATAIN, BitValue
        SetBit ADCInstructionStream(n + 1), DATAIN, BitValue
        SetBit ADCInstructionStream(n + 2), DATAIN, BitValue
        SetBit ADCInstructionStream(n + 3), DATAIN, BitValue
    Next


    'Set data length ( 2 bits ).
    BitValue = ChkBit(DataLength, 1)
    SetBit ADCInstructionStream(12), DATAIN, BitValue
    SetBit ADCInstructionStream(13), DATAIN, BitValue
    SetBit ADCInstructionStream(14), DATAIN, BitValue

    BitValue = ChkBit(DataLength, 0)
    SetBit ADCInstructionStream(15), DATAIN, BitValue
    SetBit ADCInstructionStream(16), DATAIN, BitValue
    SetBit ADCInstructionStream(17), DATAIN, BitValue


    'Set LSBF ( 1 bit ).
    BitValue = ChkBit(LSBF, 0)
    SetBit ADCInstructionStream(18), DATAIN, BitValue
    SetBit ADCInstructionStream(19), DATAIN, BitValue
    SetBit ADCInstructionStream(20), DATAIN, BitValue

    'Set BIP ( 1 bit ).
    BitValue = ChkBit(BIP, 0)
    SetBit ADCInstructionStream(21), DATAIN, BitValue
    SetBit ADCInstructionStream(22), DATAIN, BitValue
    SetBit ADCInstructionStream(23), DATAIN, BitValue


    ' Initialise the ADC by taking CS HI then LO. CS remains low until the next call to ReadVoltage().
    Out DATA, 1
    Wait 0.000001 ' Wait a bit.
    Out DATA, 0
    Wait 0.000001 ' Wait a bit for ADC to settle down.
    
    ' Send the instruction and read the result of the previous conversion.
    iIndex = 0
    For n = 0 To 35

        'Before each CLK pulse is sent, the STATUS port is read in order to fetch and store
        'the individual bits of the result of the previous conversion as it comes back from
        'the ADC.
        If (n Mod 3) = 0 Then
        
            ADCResult(iIndex) = Inp(STATUS)
            iIndex = iIndex + 1
        
        End If

        Out DATA, ADCInstructionStream(n)
        ' Delay between individual writes.
        Wait (0.000001)

    Next

    
    'Array ADCResult now holds all the STATUS port bytes read in.
    'The bit 6 column of all the bytes contains the 12-bit result.
    'Read the values of these bits and convert to the final voltage
    'reading.
    
    'Convert the individual 12 bits read in to an integer value.
    iADCResult = 0
    For n = 0 To 10
    
        iADCResult = iADCResult + ChkBit(ADCResult(n), 6)
        iADCResult = iADCResult * 2 ' Shift bits left

    Next
    
    iADCResult = iADCResult + ChkBit(ADCResult(11), 6)

    
    ' Convert the resultant ADC integer value to a real voltage.
    dVoltage = iADCResult / 4095 * 5
    
    
    ReadVoltage = dVoltage ' Return result.

    
End Function


' WriteVoltage : Write a specified voltage to one of 8 DAC output pins.

'Params :-
'Pin    : Specify pin to output a voltage ( 0-7 ).
'Range  : Specify the voltage range to output ( 0= x1, 1= x2 )
'Voltage: Specify the voltage to output ( 0V-2.5V )

'********************************************************************************************************
'
' DAC control lines: LOAD( CS=HI ),DACCLK,DATAIN
'
' DATA = 888    LPT1 port address ( OUT: Control info to DAC i.e, LOAD(CS),CLK,DATAIN )
'
' DAC 12-bit instruction format: A3,A2,A1,R,D7,D6,D5,D4,D3,D2,D1,D0 ( A=Address(Pin), R=Output voltage range, D=Voltage )
'
' For example, an instruction sent to the DAC on the DATAIN line is: 0,0,0,0,0,0,0,0,0,0,0,0
'
'               Address(3 bits)= 0,0,0 ( equates to DAC output pin 0 ).
'               Range(1 bit)= 0 ( equates to an output voltage range of times 1 ).
'               Voltage(8 bits)= 0 ( equates to outputting 0V ).
'
'********************************************************************************************************
'
' DATA port output byte format: 0,0,0,0,DATAIN,DACCLK,0,LOAD
'
' Each instruction bit sent to the DAC on DATAIN consists of a 3 byte write sequence
' that encapsulates a full CLK pulse. This is to ensure that each instruction bit
' is present on the DATAIN line BEFORE and AFTER a CLK pulse is generated.
' This is to eliminate any timing issues so that an instruction bit is properly
' captured by the DAC.
'
'**********************************************************************************************

Sub WriteVoltage(Pin As Byte, Range As Byte, Voltage As Double)

Dim BitValue As Byte
Dim VoltageBits As Byte
Dim n As Integer


    'Convert the input parameters to a DAC instruction sequence by amending the
    'previously initialised DAC data stream.

    'Set address ( 3 bits ).
    For n = 0 To 8 Step 3
        BitValue = ChkBit(Pin, 2 - n / 3)
        SetBit DACInstructionStream(n), DATAIN, BitValue
        SetBit DACInstructionStream(n + 1), DATAIN, BitValue
        SetBit DACInstructionStream(n + 2), DATAIN, BitValue
    Next
        
    
    'Set range ( 1 bit ).
    BitValue = ChkBit(Range, 0)
    SetBit DACInstructionStream(9), DATAIN, BitValue
    SetBit DACInstructionStream(10), DATAIN, BitValue
    SetBit DACInstructionStream(11), DATAIN, BitValue
    
    
    'Set voltage ( 8 bits ).
    VoltageBits = CInt(Voltage / DACREF * 255)
    For n = 12 To 35 Step 3
        BitValue = ChkBit(VoltageBits, 11 - n / 3)
        SetBit DACInstructionStream(n), DATAIN, BitValue
        SetBit DACInstructionStream(n + 1), DATAIN, BitValue
        SetBit DACInstructionStream(n + 2), DATAIN, BitValue
    Next
        

    'Write DAC instruction stream to DATA.
    For n = 0 To 36

        Out DATA, DACInstructionStream(n)
        
        ' Delay between individual writes.
        Wait (0.000001)

    Next

    
End Sub


Sub PrintADCInstructionStream()

Dim n As Integer

    Form1.txtPrintADC = ""
    Form1.txtPrintADC = Form1.txtPrintADC & "Address:" & vbCrLf
    For n = 0 To 11

        Form1.txtPrintADC = Form1.txtPrintADC & Format(n, "00") & " " & BitsToString(ADCInstructionStream(n)) & vbCrLf

    Next

    Form1.txtPrintADC = Form1.txtPrintADC & "Length:" & vbCrLf
    For n = 12 To 17
    
        Form1.txtPrintADC = Form1.txtPrintADC & Format(n, "00") & " " & BitsToString(ADCInstructionStream(n)) & vbCrLf
    
    Next
    
    Form1.txtPrintADC = Form1.txtPrintADC & "LSBF:" & vbCrLf
    For n = 18 To 20
    
        Form1.txtPrintADC = Form1.txtPrintADC & Format(n, "00") & " " & BitsToString(ADCInstructionStream(n)) & vbCrLf
    
    Next
    
    Form1.txtPrintADC = Form1.txtPrintADC & "BIP:" & vbCrLf
    For n = 21 To 23
    
        Form1.txtPrintADC = Form1.txtPrintADC & Format(n, "00") & " " & BitsToString(ADCInstructionStream(n)) & vbCrLf
    
    Next

    Form1.txtPrintADC = Form1.txtPrintADC & "Pad Bits:" & vbCrLf
    For n = 24 To 35
    
        Form1.txtPrintADC = Form1.txtPrintADC & Format(n, "00") & " " & BitsToString(ADCInstructionStream(n)) & vbCrLf
    
    Next


End Sub


Sub PrintADCResult()

Dim n As Integer

    Form1.txtPrintADCResult = ""
    For n = 0 To 11
    
        Form1.txtPrintADCResult = Form1.txtPrintADCResult & Format(n, "00") & " " & BitsToString(ADCResult(n)) & vbCrLf
'        Form1.txtPrintADCResult = Form1.txtPrintADCResult & Format(n, "00") & " " & ADCResult(n) & vbCrLf

    Next


End Sub


Sub PrintDACInstructionStream()

Dim n As Integer

    Form1.txtPrintDAC = ""
    Form1.txtPrintDAC = Form1.txtPrintDAC & "Address:" & vbCrLf
    For n = 0 To 8

        Form1.txtPrintDAC = Form1.txtPrintDAC & Format(n, "00") & " " & BitsToString(DACInstructionStream(n)) & vbCrLf

    Next

    Form1.txtPrintDAC = Form1.txtPrintDAC & "Range:" & vbCrLf
    For n = 9 To 11

        Form1.txtPrintDAC = Form1.txtPrintDAC & Format(n, "00") & " " & BitsToString(DACInstructionStream(n)) & vbCrLf

    Next

    Form1.txtPrintDAC = Form1.txtPrintDAC & "Data:" & vbCrLf
    For n = 12 To 35

        Form1.txtPrintDAC = Form1.txtPrintDAC & Format(n, "00") & " " & BitsToString(DACInstructionStream(n)) & vbCrLf

    Next


    Form1.txtPrintDAC = Form1.txtPrintDAC & "LOAD LOW:" & vbCrLf
    Form1.txtPrintDAC = Form1.txtPrintDAC & Format(n, "00") & " " & BitsToString(DACInstructionStream(36))

End Sub

